from unittest import TestCase


class TestAtom(TestCase):
    def test_set_number(self):
        self.fail()

    def test_set_coordinates(self):
        self.fail()

    def test_get_coordinates(self):
        self.fail()

    def test_set_id(self):
        self.fail()

    def test_set_occupancy(self):
        self.fail()

    def test_set_bfactor(self):
        self.fail()

    def test_set_element(self):
        self.fail()

    def test_set_residue(self):
        self.fail()
