from unittest import TestCase


class TestDNA(TestCase):
    def test_set_number(self):
        self.fail()

    def test_set_atoms(self):
        self.fail()

    def test_insert_atom(self):
        self.fail()

    def test_set_name(self):
        self.fail()

    def test_set_n(self):
        self.fail()
