from unittest import TestCase


class TestAminoAcid(TestCase):
    def test_set_number(self):
        self.fail()

    def test_set_atoms(self):
        self.fail()

    def test_insert_atom(self):
        self.fail()

    def test_insert_atom(self):
        self.fail()

    def test_get_atoms(self):
        self.fail()

    def test_set_name(self):
        self.fail()

    def test_set_heptad(self):
        self.fail()

    def test_calculate_centroid(self):
        self.fail()

    def test_get_centroid(self):
        self.fail()

    def test_one_to_three(self):
        self.fail()

    def test_name_to_three(self):
        self.fail()
