from apalib.Container import Container
import apalib.Container
from apalib.AminoAcid import AminoAcid
from apalib.Atom import Atom
from apalib.DNA import DNA
from apalib.RNA import RNA
from apalib.HETATM import HETATM

from apalib.Data import Data
Container = Container()
data = Data()