from unittest import TestCase
import __init__

class Testinit(TestCase):
    def setUp(self):
        __init__.Fetch('5u59')
        print("Setup complete")

    def test_set_number(self):
        self.fail()

    def test_set_atoms(self):
        self.fail()

    def test_insert_atom(self):
        self.fail()

    def test_set_name(self):
        self.fail()
